var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var Plugin_exports = {};
__export(Plugin_exports, {
  default: () => WorkflowPlugin
});
module.exports = __toCommonJS(Plugin_exports);
var import_path = __toESM(require("path"));
var import_lru_cache = __toESM(require("lru-cache"));
var import_winston = __toESM(require("winston"));
var import_database = require("@nocobase/database");
var import_server = require("@nocobase/server");
var import_utils = require("@nocobase/utils");
var import_logger = require("@nocobase/logger");
var import_Processor = __toESM(require("./Processor"));
var import_actions = __toESM(require("./actions"));
var import_constants = require("./constants");
var import_functions = __toESM(require("./functions"));
var import_CollectionTrigger = __toESM(require("./triggers/CollectionTrigger"));
var import_ScheduleTrigger = __toESM(require("./triggers/ScheduleTrigger"));
var import_CalculationInstruction = __toESM(require("./instructions/CalculationInstruction"));
var import_ConditionInstruction = __toESM(require("./instructions/ConditionInstruction"));
var import_CreateInstruction = __toESM(require("./instructions/CreateInstruction"));
var import_DestroyInstruction = __toESM(require("./instructions/DestroyInstruction"));
var import_QueryInstruction = __toESM(require("./instructions/QueryInstruction"));
var import_UpdateInstruction = __toESM(require("./instructions/UpdateInstruction"));
class WorkflowPlugin extends import_server.Plugin {
  instructions = new import_utils.Registry();
  triggers = new import_utils.Registry();
  functions = new import_utils.Registry();
  ready = false;
  executing = null;
  pending = [];
  events = [];
  loggerCache;
  getLogger(workflowId) {
    const now = /* @__PURE__ */ new Date();
    const date = `${now.getFullYear()}-${`0${now.getMonth() + 1}`.slice(-2)}-${`0${now.getDate()}`.slice(-2)}`;
    const key = `${date}-${workflowId}}`;
    if (this.loggerCache.has(key)) {
      return this.loggerCache.get(key);
    }
    const logger = (0, import_logger.createLogger)({
      transports: [
        ...process.env.NODE_ENV !== "production" ? ["console"] : [],
        new import_winston.default.transports.File({
          filename: (0, import_logger.getLoggerFilePath)("workflows", date, `${workflowId}.log`),
          level: (0, import_logger.getLoggerLevel)()
        })
      ]
    });
    this.loggerCache.set(key, logger);
    return logger;
  }
  onBeforeSave = async (instance, options) => {
    const Model = instance.constructor;
    if (instance.enabled) {
      instance.set("current", true);
    } else if (!instance.current) {
      const count = await Model.count({
        where: {
          key: instance.key
        },
        transaction: options.transaction
      });
      if (!count) {
        instance.set("current", true);
      }
    }
    if (!instance.changed("enabled") || !instance.enabled) {
      return;
    }
    const previous = await Model.findOne({
      where: {
        key: instance.key,
        current: true,
        id: {
          [import_database.Op.ne]: instance.id
        }
      },
      transaction: options.transaction
    });
    if (previous) {
      await previous.update(
        { enabled: false, current: null },
        {
          transaction: options.transaction,
          hooks: false
        }
      );
      this.toggle(previous, false);
    }
  };
  initTriggers(more = {}) {
    const { triggers } = this;
    triggers.register("collection", new import_CollectionTrigger.default(this));
    triggers.register("schedule", new import_ScheduleTrigger.default(this));
    for (const [name, trigger] of Object.entries(more)) {
      triggers.register(name, typeof trigger === "function" ? new trigger(this) : trigger);
    }
  }
  initInstructions(more = {}) {
    const { instructions } = this;
    instructions.register("calculation", new import_CalculationInstruction.default(this));
    instructions.register("condition", new import_ConditionInstruction.default(this));
    instructions.register("create", new import_CreateInstruction.default(this));
    instructions.register("destroy", new import_DestroyInstruction.default(this));
    instructions.register("query", new import_QueryInstruction.default(this));
    instructions.register("update", new import_UpdateInstruction.default(this));
    for (const [name, instruction] of Object.entries({ ...more })) {
      instructions.register(name, typeof instruction === "function" ? new instruction(this) : instruction);
    }
  }
  async load() {
    const { db, options } = this;
    (0, import_actions.default)(this);
    this.initTriggers(options.triggers);
    this.initInstructions(options.instructions);
    (0, import_functions.default)(this, options.functions);
    this.loggerCache = new import_lru_cache.default({
      max: 20,
      updateAgeOnGet: true,
      dispose(logger) {
        logger.end();
      }
    });
    this.app.acl.registerSnippet({
      name: `pm.${this.name}.workflows`,
      actions: [
        "workflows:*",
        "workflows.nodes:*",
        "executions:list",
        "executions:get",
        "flow_nodes:update",
        "flow_nodes:destroy"
      ]
    });
    this.app.acl.registerSnippet({
      name: "ui.*",
      actions: ["workflows:list"]
    });
    this.app.acl.allow("users_jobs", ["list", "get", "submit"], "loggedIn");
    this.app.acl.allow("workflows", ["trigger"], "loggedIn");
    await db.import({
      directory: import_path.default.resolve(__dirname, "collections")
    });
    this.db.addMigrations({
      namespace: this.name,
      directory: import_path.default.resolve(__dirname, "migrations"),
      context: {
        plugin: this
      }
    });
    db.on("workflows.beforeSave", this.onBeforeSave);
    db.on("workflows.afterSave", (model) => this.toggle(model));
    db.on("workflows.afterDestroy", (model) => this.toggle(model, false));
    this.app.on("beforeStart", async () => {
      const collection = db.getCollection("workflows");
      const workflows = await collection.repository.find({
        filter: { enabled: true }
      });
      workflows.forEach((workflow) => {
        this.toggle(workflow);
      });
    });
    this.app.on("afterStart", () => {
      this.app.setMaintainingMessage("check for not started executions");
      this.ready = true;
      this.dispatch();
    });
    this.app.on("beforeStop", async () => {
      const collection = db.getCollection("workflows");
      const workflows = await collection.repository.find({
        filter: { enabled: true }
      });
      workflows.forEach((workflow) => {
        this.toggle(workflow, false);
      });
      this.ready = false;
      await this.prepare();
      if (this.executing) {
        await this.executing;
      }
    });
  }
  toggle(workflow, enable) {
    const type = workflow.get("type");
    const trigger = this.triggers.get(type);
    if (typeof enable !== "undefined" ? enable : workflow.get("enabled")) {
      const prev = workflow.previous();
      if (prev.config) {
        trigger.off({ ...workflow.get(), ...prev });
      }
      trigger.on(workflow);
    } else {
      trigger.off(workflow);
    }
  }
  trigger(workflow, context, options = {}) {
    if (!this.ready || context == null) {
      return;
    }
    this.events.push([workflow, context, options]);
    this.getLogger(workflow.id).info(`new event triggered, now events: ${this.events.length}`);
    this.getLogger(workflow.id).debug(`event data:`, {
      data: context
    });
    if (this.events.length > 1) {
      return;
    }
    setTimeout(this.prepare);
  }
  async resume(job) {
    if (!job.execution) {
      job.execution = await job.getExecution();
    }
    this.pending.push([job.execution, job]);
    this.dispatch();
  }
  createProcessor(execution, options = {}) {
    return new import_Processor.default(execution, { ...options, plugin: this });
  }
  prepare = async () => {
    var _a;
    const [event] = this.events;
    if (!event) {
      return;
    }
    const [workflow, context, options] = event;
    let valid = true;
    if ((_a = options.context) == null ? void 0 : _a.executionId) {
      const existed = await workflow.countExecutions({
        where: {
          id: options.context.executionId
        }
      });
      if (existed) {
        this.getLogger(workflow.id).warn(
          `workflow ${workflow.id} has already been triggered in same execution (${options.context.executionId}), and newly triggering will be skipped.`
        );
        valid = false;
      }
    }
    if (valid) {
      const execution = await this.db.sequelize.transaction(async (transaction) => {
        const execution2 = await workflow.createExecution(
          {
            context,
            key: workflow.key,
            status: import_constants.EXECUTION_STATUS.QUEUEING
          },
          { transaction }
        );
        await workflow.increment(["executed", "allExecuted"], { transaction });
        if (this.db.options.dialect !== "postgres") {
          await workflow.reload({ transaction });
        }
        await workflow.constructor.update(
          {
            allExecuted: workflow.allExecuted
          },
          {
            where: {
              key: workflow.key
            },
            transaction
          }
        );
        execution2.workflow = workflow;
        return execution2;
      });
      this.getLogger(workflow.id).debug(`execution of workflow ${workflow.id} created as ${execution.id}`, {
        data: execution.context
      });
      if (!this.executing && !this.pending.length) {
        this.pending.push([execution]);
      }
    }
    this.events.shift();
    if (this.events.length) {
      await this.prepare();
    } else {
      this.dispatch();
    }
  };
  dispatch() {
    if (!this.ready || this.executing) {
      return;
    }
    this.executing = (async () => {
      let next = null;
      if (this.pending.length) {
        next = this.pending.shift();
        this.getLogger(next[0].workflowId).info(`pending execution (${next[0].id}) ready to process`);
      } else {
        const execution = await this.db.getRepository("executions").findOne({
          filter: {
            status: import_constants.EXECUTION_STATUS.QUEUEING,
            "workflow.id": {
              [import_database.Op.not]: null
            }
          },
          appends: ["workflow"],
          sort: "createdAt"
        });
        if (execution && execution.workflow.enabled) {
          this.getLogger(execution.workflowId).info(`execution (${execution.id}) fetched from db`);
          next = [execution];
        }
      }
      if (next) {
        await this.process(...next);
      }
      this.executing = null;
      if (next) {
        this.dispatch();
      }
    })();
  }
  async process(execution, job) {
    var _a, _b;
    if (execution.status === import_constants.EXECUTION_STATUS.QUEUEING) {
      await execution.update({ status: import_constants.EXECUTION_STATUS.STARTED });
    }
    const processor = this.createProcessor(execution);
    this.getLogger(execution.workflowId).info(`execution (${execution.id}) ${job ? "resuming" : "starting"}...`);
    try {
      await (job ? processor.resume(job) : processor.start());
      this.getLogger(execution.workflowId).info(
        `execution (${execution.id}) finished with status: ${execution.status}`
      );
      if (execution.status && ((_b = (_a = execution.workflow.options) == null ? void 0 : _a.deleteExecutionOnStatus) == null ? void 0 : _b.includes(execution.status))) {
        await execution.destroy();
      }
    } catch (err) {
      this.getLogger(execution.workflowId).error(`execution (${execution.id}) error: ${err.message}`, err);
    }
  }
}
